# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class DeviceCodeList(ListResource):

    def __init__(self, version):
        """
        Initialize the DeviceCodeList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.oauth.v1.device_code.DeviceCodeList
        :rtype: twilio.rest.oauth.v1.device_code.DeviceCodeList
        """
        super(DeviceCodeList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/device/code'.format(**self._solution)

    def create(self, client_sid, scopes, audiences=values.unset):
        """
        Create the DeviceCodeInstance

        :param unicode client_sid: A string that uniquely identifies this oauth app
        :param list[unicode] scopes: An Array of scopes
        :param list[unicode] audiences: An array of intended audiences

        :returns: The created DeviceCodeInstance
        :rtype: twilio.rest.oauth.v1.device_code.DeviceCodeInstance
        """
        data = values.of({
            'ClientSid': client_sid,
            'Scopes': serialize.map(scopes, lambda e: e),
            'Audiences': serialize.map(audiences, lambda e: e),
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return DeviceCodeInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.DeviceCodeList>'


class DeviceCodePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the DeviceCodePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.oauth.v1.device_code.DeviceCodePage
        :rtype: twilio.rest.oauth.v1.device_code.DeviceCodePage
        """
        super(DeviceCodePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of DeviceCodeInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.oauth.v1.device_code.DeviceCodeInstance
        :rtype: twilio.rest.oauth.v1.device_code.DeviceCodeInstance
        """
        return DeviceCodeInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.DeviceCodePage>'


class DeviceCodeInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the DeviceCodeInstance

        :returns: twilio.rest.oauth.v1.device_code.DeviceCodeInstance
        :rtype: twilio.rest.oauth.v1.device_code.DeviceCodeInstance
        """
        super(DeviceCodeInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'device_code': payload.get('device_code'),
            'user_code': payload.get('user_code'),
            'verification_uri': payload.get('verification_uri'),
            'verification_uri_complete': payload.get('verification_uri_complete'),
            'expires_in': deserialize.integer(payload.get('expires_in')),
            'interval': deserialize.integer(payload.get('interval')),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def device_code(self):
        """
        :returns: The device verification code
        :rtype: unicode
        """
        return self._properties['device_code']

    @property
    def user_code(self):
        """
        :returns: The verification code for the end user
        :rtype: unicode
        """
        return self._properties['user_code']

    @property
    def verification_uri(self):
        """
        :returns: The URI that the end user visits to verify request
        :rtype: unicode
        """
        return self._properties['verification_uri']

    @property
    def verification_uri_complete(self):
        """
        :returns: he URI with user_code that the end-user alternatively visits to verify request
        :rtype: unicode
        """
        return self._properties['verification_uri_complete']

    @property
    def expires_in(self):
        """
        :returns: The expiration time of the device_code and user_code in seconds
        :rtype: unicode
        """
        return self._properties['expires_in']

    @property
    def interval(self):
        """
        :returns: The minimum amount of time in seconds that the client should wait between polling requests to the token endpoint
        :rtype: unicode
        """
        return self._properties['interval']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.DeviceCodeInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.sync.v1.service.document import DocumentList
from twilio.rest.sync.v1.service.sync_list import SyncListList
from twilio.rest.sync.v1.service.sync_map import SyncMapList
from twilio.rest.sync.v1.service.sync_stream import SyncStreamList


class ServiceList(ListResource):

    def __init__(self, version):
        """
        Initialize the ServiceList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.sync.v1.service.ServiceList
        :rtype: twilio.rest.sync.v1.service.ServiceList
        """
        super(ServiceList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Services'.format(**self._solution)

    def create(self, friendly_name=values.unset, webhook_url=values.unset,
               reachability_webhooks_enabled=values.unset, acl_enabled=values.unset,
               reachability_debouncing_enabled=values.unset,
               reachability_debouncing_window=values.unset,
               webhooks_from_rest_enabled=values.unset):
        """
        Create the ServiceInstance

        :param unicode friendly_name: A string that you assign to describe the resource
        :param unicode webhook_url: The URL we should call when Sync objects are manipulated
        :param bool reachability_webhooks_enabled: Whether the service instance should call webhook_url when client endpoints connect to Sync
        :param bool acl_enabled: Whether token identities in the Service must be granted access to Sync objects by using the Permissions resource
        :param bool reachability_debouncing_enabled: Whether every endpoint_disconnected event occurs after a configurable delay
        :param unicode reachability_debouncing_window: The reachability event delay in milliseconds
        :param bool webhooks_from_rest_enabled: Whether the Service instance should call webhook_url when the REST API is used to update Sync objects

        :returns: The created ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServiceInstance
        """
        data = values.of({
            'FriendlyName': friendly_name,
            'WebhookUrl': webhook_url,
            'ReachabilityWebhooksEnabled': reachability_webhooks_enabled,
            'AclEnabled': acl_enabled,
            'ReachabilityDebouncingEnabled': reachability_debouncing_enabled,
            'ReachabilityDebouncingWindow': reachability_debouncing_window,
            'WebhooksFromRestEnabled': webhooks_from_rest_enabled,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return ServiceInstance(self._version, payload, )

    def stream(self, limit=None, page_size=None):
        """
        Streams ServiceInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.sync.v1.service.ServiceInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists ServiceInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.sync.v1.service.ServiceInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of ServiceInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServicePage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return ServicePage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of ServiceInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServicePage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return ServicePage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a ServiceContext

        :param sid: The SID of the Service resource to fetch

        :returns: twilio.rest.sync.v1.service.ServiceContext
        :rtype: twilio.rest.sync.v1.service.ServiceContext
        """
        return ServiceContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a ServiceContext

        :param sid: The SID of the Service resource to fetch

        :returns: twilio.rest.sync.v1.service.ServiceContext
        :rtype: twilio.rest.sync.v1.service.ServiceContext
        """
        return ServiceContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Sync.V1.ServiceList>'


class ServicePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the ServicePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.sync.v1.service.ServicePage
        :rtype: twilio.rest.sync.v1.service.ServicePage
        """
        super(ServicePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of ServiceInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.sync.v1.service.ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServiceInstance
        """
        return ServiceInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Sync.V1.ServicePage>'


class ServiceContext(InstanceContext):

    def __init__(self, version, sid):
        """
        Initialize the ServiceContext

        :param Version version: Version that contains the resource
        :param sid: The SID of the Service resource to fetch

        :returns: twilio.rest.sync.v1.service.ServiceContext
        :rtype: twilio.rest.sync.v1.service.ServiceContext
        """
        super(ServiceContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Services/{sid}'.format(**self._solution)

        # Dependents
        self._documents = None
        self._sync_lists = None
        self._sync_maps = None
        self._sync_streams = None

    def fetch(self):
        """
        Fetch the ServiceInstance

        :returns: The fetched ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServiceInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return ServiceInstance(self._version, payload, sid=self._solution['sid'], )

    def delete(self):
        """
        Deletes the ServiceInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def update(self, webhook_url=values.unset, friendly_name=values.unset,
               reachability_webhooks_enabled=values.unset, acl_enabled=values.unset,
               reachability_debouncing_enabled=values.unset,
               reachability_debouncing_window=values.unset,
               webhooks_from_rest_enabled=values.unset):
        """
        Update the ServiceInstance

        :param unicode webhook_url: The URL we should call when Sync objects are manipulated
        :param unicode friendly_name: A string that you assign to describe the resource
        :param bool reachability_webhooks_enabled: Whether the service instance should call webhook_url when client endpoints connect to Sync
        :param bool acl_enabled: Whether token identities in the Service must be granted access to Sync objects by using the Permissions resource
        :param bool reachability_debouncing_enabled: Whether every endpoint_disconnected event occurs after a configurable delay
        :param unicode reachability_debouncing_window: The reachability event delay in milliseconds
        :param bool webhooks_from_rest_enabled: Whether the Service instance should call webhook_url when the REST API is used to update Sync objects

        :returns: The updated ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServiceInstance
        """
        data = values.of({
            'WebhookUrl': webhook_url,
            'FriendlyName': friendly_name,
            'ReachabilityWebhooksEnabled': reachability_webhooks_enabled,
            'AclEnabled': acl_enabled,
            'ReachabilityDebouncingEnabled': reachability_debouncing_enabled,
            'ReachabilityDebouncingWindow': reachability_debouncing_window,
            'WebhooksFromRestEnabled': webhooks_from_rest_enabled,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return ServiceInstance(self._version, payload, sid=self._solution['sid'], )

    @property
    def documents(self):
        """
        Access the documents

        :returns: twilio.rest.sync.v1.service.document.DocumentList
        :rtype: twilio.rest.sync.v1.service.document.DocumentList
        """
        if self._documents is None:
            self._documents = DocumentList(self._version, service_sid=self._solution['sid'], )
        return self._documents

    @property
    def sync_lists(self):
        """
        Access the sync_lists

        :returns: twilio.rest.sync.v1.service.sync_list.SyncListList
        :rtype: twilio.rest.sync.v1.service.sync_list.SyncListList
        """
        if self._sync_lists is None:
            self._sync_lists = SyncListList(self._version, service_sid=self._solution['sid'], )
        return self._sync_lists

    @property
    def sync_maps(self):
        """
        Access the sync_maps

        :returns: twilio.rest.sync.v1.service.sync_map.SyncMapList
        :rtype: twilio.rest.sync.v1.service.sync_map.SyncMapList
        """
        if self._sync_maps is None:
            self._sync_maps = SyncMapList(self._version, service_sid=self._solution['sid'], )
        return self._sync_maps

    @property
    def sync_streams(self):
        """
        Access the sync_streams

        :returns: twilio.rest.sync.v1.service.sync_stream.SyncStreamList
        :rtype: twilio.rest.sync.v1.service.sync_stream.SyncStreamList
        """
        if self._sync_streams is None:
            self._sync_streams = SyncStreamList(self._version, service_sid=self._solution['sid'], )
        return self._sync_streams

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Sync.V1.ServiceContext {}>'.format(context)


class ServiceInstance(InstanceResource):

    def __init__(self, version, payload, sid=None):
        """
        Initialize the ServiceInstance

        :returns: twilio.rest.sync.v1.service.ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServiceInstance
        """
        super(ServiceInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'unique_name': payload.get('unique_name'),
            'account_sid': payload.get('account_sid'),
            'friendly_name': payload.get('friendly_name'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
            'webhook_url': payload.get('webhook_url'),
            'webhooks_from_rest_enabled': payload.get('webhooks_from_rest_enabled'),
            'reachability_webhooks_enabled': payload.get('reachability_webhooks_enabled'),
            'acl_enabled': payload.get('acl_enabled'),
            'reachability_debouncing_enabled': payload.get('reachability_debouncing_enabled'),
            'reachability_debouncing_window': deserialize.integer(payload.get('reachability_debouncing_window')),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: ServiceContext for this ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServiceContext
        """
        if self._context is None:
            self._context = ServiceContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def unique_name(self):
        """
        :returns: An application-defined string that uniquely identifies the resource
        :rtype: unicode
        """
        return self._properties['unique_name']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def friendly_name(self):
        """
        :returns: The string that you assigned to describe the resource
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The absolute URL of the Service resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def webhook_url(self):
        """
        :returns: The URL we call when Sync objects are manipulated
        :rtype: unicode
        """
        return self._properties['webhook_url']

    @property
    def webhooks_from_rest_enabled(self):
        """
        :returns: Whether the Service instance should call webhook_url when the REST API is used to update Sync objects
        :rtype: bool
        """
        return self._properties['webhooks_from_rest_enabled']

    @property
    def reachability_webhooks_enabled(self):
        """
        :returns: Whether the service instance calls webhook_url when client endpoints connect to Sync
        :rtype: bool
        """
        return self._properties['reachability_webhooks_enabled']

    @property
    def acl_enabled(self):
        """
        :returns: Whether token identities in the Service must be granted access to Sync objects by using the Permissions resource
        :rtype: bool
        """
        return self._properties['acl_enabled']

    @property
    def reachability_debouncing_enabled(self):
        """
        :returns: Whether every endpoint_disconnected event occurs after a configurable delay
        :rtype: bool
        """
        return self._properties['reachability_debouncing_enabled']

    @property
    def reachability_debouncing_window(self):
        """
        :returns: The reachability event delay in milliseconds
        :rtype: unicode
        """
        return self._properties['reachability_debouncing_window']

    @property
    def links(self):
        """
        :returns: The URLs of related resources
        :rtype: unicode
        """
        return self._properties['links']

    def fetch(self):
        """
        Fetch the ServiceInstance

        :returns: The fetched ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServiceInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the ServiceInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def update(self, webhook_url=values.unset, friendly_name=values.unset,
               reachability_webhooks_enabled=values.unset, acl_enabled=values.unset,
               reachability_debouncing_enabled=values.unset,
               reachability_debouncing_window=values.unset,
               webhooks_from_rest_enabled=values.unset):
        """
        Update the ServiceInstance

        :param unicode webhook_url: The URL we should call when Sync objects are manipulated
        :param unicode friendly_name: A string that you assign to describe the resource
        :param bool reachability_webhooks_enabled: Whether the service instance should call webhook_url when client endpoints connect to Sync
        :param bool acl_enabled: Whether token identities in the Service must be granted access to Sync objects by using the Permissions resource
        :param bool reachability_debouncing_enabled: Whether every endpoint_disconnected event occurs after a configurable delay
        :param unicode reachability_debouncing_window: The reachability event delay in milliseconds
        :param bool webhooks_from_rest_enabled: Whether the Service instance should call webhook_url when the REST API is used to update Sync objects

        :returns: The updated ServiceInstance
        :rtype: twilio.rest.sync.v1.service.ServiceInstance
        """
        return self._proxy.update(
            webhook_url=webhook_url,
            friendly_name=friendly_name,
            reachability_webhooks_enabled=reachability_webhooks_enabled,
            acl_enabled=acl_enabled,
            reachability_debouncing_enabled=reachability_debouncing_enabled,
            reachability_debouncing_window=reachability_debouncing_window,
            webhooks_from_rest_enabled=webhooks_from_rest_enabled,
        )

    @property
    def documents(self):
        """
        Access the documents

        :returns: twilio.rest.sync.v1.service.document.DocumentList
        :rtype: twilio.rest.sync.v1.service.document.DocumentList
        """
        return self._proxy.documents

    @property
    def sync_lists(self):
        """
        Access the sync_lists

        :returns: twilio.rest.sync.v1.service.sync_list.SyncListList
        :rtype: twilio.rest.sync.v1.service.sync_list.SyncListList
        """
        return self._proxy.sync_lists

    @property
    def sync_maps(self):
        """
        Access the sync_maps

        :returns: twilio.rest.sync.v1.service.sync_map.SyncMapList
        :rtype: twilio.rest.sync.v1.service.sync_map.SyncMapList
        """
        return self._proxy.sync_maps

    @property
    def sync_streams(self):
        """
        Access the sync_streams

        :returns: twilio.rest.sync.v1.service.sync_stream.SyncStreamList
        :rtype: twilio.rest.sync.v1.service.sync_stream.SyncStreamList
        """
        return self._proxy.sync_streams

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Sync.V1.ServiceInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class ApprovalFetchList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, sid):
        """
        Initialize the ApprovalFetchList

        :param Version version: Version that contains the resource
        :param sid: The unique string that identifies the Content resource

        :returns: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchList
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchList
        """
        super(ApprovalFetchList, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }

    def get(self):
        """
        Constructs a ApprovalFetchContext

        :returns: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchContext
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchContext
        """
        return ApprovalFetchContext(self._version, sid=self._solution['sid'], )

    def __call__(self):
        """
        Constructs a ApprovalFetchContext

        :returns: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchContext
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchContext
        """
        return ApprovalFetchContext(self._version, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Content.V1.ApprovalFetchList>'


class ApprovalFetchPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the ApprovalFetchPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param sid: The unique string that identifies the Content resource

        :returns: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchPage
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchPage
        """
        super(ApprovalFetchPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of ApprovalFetchInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchInstance
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchInstance
        """
        return ApprovalFetchInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Content.V1.ApprovalFetchPage>'


class ApprovalFetchContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, sid):
        """
        Initialize the ApprovalFetchContext

        :param Version version: Version that contains the resource
        :param sid: The unique string that identifies the Content resource

        :returns: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchContext
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchContext
        """
        super(ApprovalFetchContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Content/{sid}/ApprovalRequests'.format(**self._solution)

    def fetch(self):
        """
        Fetch the ApprovalFetchInstance

        :returns: The fetched ApprovalFetchInstance
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return ApprovalFetchInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Content.V1.ApprovalFetchContext {}>'.format(context)


class ApprovalFetchInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload, sid):
        """
        Initialize the ApprovalFetchInstance

        :returns: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchInstance
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchInstance
        """
        super(ApprovalFetchInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'whatsapp': payload.get('whatsapp'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid, }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: ApprovalFetchContext for this ApprovalFetchInstance
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchContext
        """
        if self._context is None:
            self._context = ApprovalFetchContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the Content resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the Content resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def whatsapp(self):
        """
        :returns: Contains the whatsapp approval information for the Content resource
        :rtype: dict
        """
        return self._properties['whatsapp']

    @property
    def url(self):
        """
        :returns: The URL of the resource, relative to `https://content.twilio.com`
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the ApprovalFetchInstance

        :returns: The fetched ApprovalFetchInstance
        :rtype: twilio.rest.content.v1.content.approval_fetch.ApprovalFetchInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Content.V1.ApprovalFetchInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class InsightsSettingsCommentList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version):
        """
        Initialize the InsightsSettingsCommentList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentList
        :rtype: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentList
        """
        super(InsightsSettingsCommentList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Insights/QM/Settings/CommentTags'.format(**self._solution)

    def fetch(self, token=values.unset):
        """
        Fetch the InsightsSettingsCommentInstance

        :param unicode token: The Token HTTP request header

        :returns: The fetched InsightsSettingsCommentInstance
        :rtype: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentInstance
        """
        headers = values.of({'Token': token, })

        payload = self._version.fetch(method='GET', uri=self._uri, headers=headers, )

        return InsightsSettingsCommentInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InsightsSettingsCommentList>'


class InsightsSettingsCommentPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the InsightsSettingsCommentPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentPage
        :rtype: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentPage
        """
        super(InsightsSettingsCommentPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of InsightsSettingsCommentInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentInstance
        :rtype: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentInstance
        """
        return InsightsSettingsCommentInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InsightsSettingsCommentPage>'


class InsightsSettingsCommentInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload):
        """
        Initialize the InsightsSettingsCommentInstance

        :returns: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentInstance
        :rtype: twilio.rest.flex_api.v1.insights_settings_comment.InsightsSettingsCommentInstance
        """
        super(InsightsSettingsCommentInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'comments': payload.get('comments'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource and owns this Flex Insights
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def comments(self):
        """
        :returns: The comments
        :rtype: dict
        """
        return self._properties['comments']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InsightsSettingsCommentInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.serverless.v1.service.build.build_status import BuildStatusList


class BuildList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, service_sid):
        """
        Initialize the BuildList

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the Service that the Build resource is associated with

        :returns: twilio.rest.serverless.v1.service.build.BuildList
        :rtype: twilio.rest.serverless.v1.service.build.BuildList
        """
        super(BuildList, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, }
        self._uri = '/Services/{service_sid}/Builds'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams BuildInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.serverless.v1.service.build.BuildInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists BuildInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.serverless.v1.service.build.BuildInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of BuildInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of BuildInstance
        :rtype: twilio.rest.serverless.v1.service.build.BuildPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return BuildPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of BuildInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of BuildInstance
        :rtype: twilio.rest.serverless.v1.service.build.BuildPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return BuildPage(self._version, response, self._solution)

    def create(self, asset_versions=values.unset, function_versions=values.unset,
               dependencies=values.unset, runtime=values.unset):
        """
        Create the BuildInstance

        :param list[unicode] asset_versions: The list of Asset Version resource SIDs to include in the Build
        :param list[unicode] function_versions: The list of the Function Version resource SIDs to include in the Build
        :param unicode dependencies: A list of objects that describe the Dependencies included in the Build
        :param unicode runtime: The Runtime version that will be used to run the Build.

        :returns: The created BuildInstance
        :rtype: twilio.rest.serverless.v1.service.build.BuildInstance
        """
        data = values.of({
            'AssetVersions': serialize.map(asset_versions, lambda e: e),
            'FunctionVersions': serialize.map(function_versions, lambda e: e),
            'Dependencies': dependencies,
            'Runtime': runtime,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return BuildInstance(self._version, payload, service_sid=self._solution['service_sid'], )

    def get(self, sid):
        """
        Constructs a BuildContext

        :param sid: The SID of the Build resource to fetch

        :returns: twilio.rest.serverless.v1.service.build.BuildContext
        :rtype: twilio.rest.serverless.v1.service.build.BuildContext
        """
        return BuildContext(self._version, service_sid=self._solution['service_sid'], sid=sid, )

    def __call__(self, sid):
        """
        Constructs a BuildContext

        :param sid: The SID of the Build resource to fetch

        :returns: twilio.rest.serverless.v1.service.build.BuildContext
        :rtype: twilio.rest.serverless.v1.service.build.BuildContext
        """
        return BuildContext(self._version, service_sid=self._solution['service_sid'], sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Serverless.V1.BuildList>'


class BuildPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the BuildPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param service_sid: The SID of the Service that the Build resource is associated with

        :returns: twilio.rest.serverless.v1.service.build.BuildPage
        :rtype: twilio.rest.serverless.v1.service.build.BuildPage
        """
        super(BuildPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of BuildInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.serverless.v1.service.build.BuildInstance
        :rtype: twilio.rest.serverless.v1.service.build.BuildInstance
        """
        return BuildInstance(self._version, payload, service_sid=self._solution['service_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Serverless.V1.BuildPage>'


class BuildContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, service_sid, sid):
        """
        Initialize the BuildContext

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the Service to fetch the Build resource from
        :param sid: The SID of the Build resource to fetch

        :returns: twilio.rest.serverless.v1.service.build.BuildContext
        :rtype: twilio.rest.serverless.v1.service.build.BuildContext
        """
        super(BuildContext, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'sid': sid, }
        self._uri = '/Services/{service_sid}/Builds/{sid}'.format(**self._solution)

        # Dependents
        self._build_status = None

    def fetch(self):
        """
        Fetch the BuildInstance

        :returns: The fetched BuildInstance
        :rtype: twilio.rest.serverless.v1.service.build.BuildInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return BuildInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the BuildInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    @property
    def build_status(self):
        """
        Access the build_status

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusList
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusList
        """
        if self._build_status is None:
            self._build_status = BuildStatusList(
                self._version,
                service_sid=self._solution['service_sid'],
                sid=self._solution['sid'],
            )
        return self._build_status

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Serverless.V1.BuildContext {}>'.format(context)


class BuildInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class Status(object):
        BUILDING = "building"
        COMPLETED = "completed"
        FAILED = "failed"

    class Runtime(object):
        NODE8 = "node8"
        NODE10 = "node10"
        NODE12 = "node12"
        NODE14 = "node14"
        NODE16 = "node16"

    def __init__(self, version, payload, service_sid, sid=None):
        """
        Initialize the BuildInstance

        :returns: twilio.rest.serverless.v1.service.build.BuildInstance
        :rtype: twilio.rest.serverless.v1.service.build.BuildInstance
        """
        super(BuildInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'service_sid': payload.get('service_sid'),
            'status': payload.get('status'),
            'asset_versions': payload.get('asset_versions'),
            'function_versions': payload.get('function_versions'),
            'dependencies': payload.get('dependencies'),
            'runtime': payload.get('runtime'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'service_sid': service_sid, 'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: BuildContext for this BuildInstance
        :rtype: twilio.rest.serverless.v1.service.build.BuildContext
        """
        if self._context is None:
            self._context = BuildContext(
                self._version,
                service_sid=self._solution['service_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the Build resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the Build resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def service_sid(self):
        """
        :returns: The SID of the Service that the Build resource is associated with
        :rtype: unicode
        """
        return self._properties['service_sid']

    @property
    def status(self):
        """
        :returns: The status of the Build
        :rtype: BuildInstance.Status
        """
        return self._properties['status']

    @property
    def asset_versions(self):
        """
        :returns: The list of Asset Version resource SIDs that are included in the Build
        :rtype: list[dict]
        """
        return self._properties['asset_versions']

    @property
    def function_versions(self):
        """
        :returns: The list of Function Version resource SIDs that are included in the Build
        :rtype: list[dict]
        """
        return self._properties['function_versions']

    @property
    def dependencies(self):
        """
        :returns: A list of objects that describe the Dependencies included in the Build
        :rtype: list[dict]
        """
        return self._properties['dependencies']

    @property
    def runtime(self):
        """
        :returns: The Runtime version that will be used to run the Build.
        :rtype: BuildInstance.Runtime
        """
        return self._properties['runtime']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the Build resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the Build resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The absolute URL of the Build resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: The links
        :rtype: unicode
        """
        return self._properties['links']

    def fetch(self):
        """
        Fetch the BuildInstance

        :returns: The fetched BuildInstance
        :rtype: twilio.rest.serverless.v1.service.build.BuildInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the BuildInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    @property
    def build_status(self):
        """
        Access the build_status

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusList
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusList
        """
        return self._proxy.build_status

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Serverless.V1.BuildInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class DomainConfigList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the DomainConfigList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.messaging.v1.domain_config.DomainConfigList
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigList
        """
        super(DomainConfigList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, domain_sid):
        """
        Constructs a DomainConfigContext

        :param domain_sid: Unique string used to identify the domain that this config should be associated with.

        :returns: twilio.rest.messaging.v1.domain_config.DomainConfigContext
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigContext
        """
        return DomainConfigContext(self._version, domain_sid=domain_sid, )

    def __call__(self, domain_sid):
        """
        Constructs a DomainConfigContext

        :param domain_sid: Unique string used to identify the domain that this config should be associated with.

        :returns: twilio.rest.messaging.v1.domain_config.DomainConfigContext
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigContext
        """
        return DomainConfigContext(self._version, domain_sid=domain_sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1.DomainConfigList>'


class DomainConfigPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the DomainConfigPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.messaging.v1.domain_config.DomainConfigPage
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigPage
        """
        super(DomainConfigPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of DomainConfigInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.messaging.v1.domain_config.DomainConfigInstance
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigInstance
        """
        return DomainConfigInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1.DomainConfigPage>'


class DomainConfigContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, domain_sid):
        """
        Initialize the DomainConfigContext

        :param Version version: Version that contains the resource
        :param domain_sid: Unique string used to identify the domain that this config should be associated with.

        :returns: twilio.rest.messaging.v1.domain_config.DomainConfigContext
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigContext
        """
        super(DomainConfigContext, self).__init__(version)

        # Path Solution
        self._solution = {'domain_sid': domain_sid, }
        self._uri = '/LinkShortening/Domains/{domain_sid}/Config'.format(**self._solution)

    def update(self, messaging_service_sids, fallback_url=values.unset,
               callback_url=values.unset,
               messaging_service_sids_action=values.unset):
        """
        Update the DomainConfigInstance

        :param list[unicode] messaging_service_sids: A list of messagingServiceSids (with prefix MG)
        :param unicode fallback_url: We will redirect requests to urls we are unable to identify to this url.
        :param unicode callback_url: URL to receive click events to your webhook whenever the recipients click on the shortened links
        :param unicode messaging_service_sids_action: An action type for messaging_service_sids operation (ADD, DELETE, REPLACE)

        :returns: The updated DomainConfigInstance
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigInstance
        """
        data = values.of({
            'MessagingServiceSids': serialize.map(messaging_service_sids, lambda e: e),
            'FallbackUrl': fallback_url,
            'CallbackUrl': callback_url,
            'MessagingServiceSidsAction': messaging_service_sids_action,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return DomainConfigInstance(self._version, payload, domain_sid=self._solution['domain_sid'], )

    def fetch(self):
        """
        Fetch the DomainConfigInstance

        :returns: The fetched DomainConfigInstance
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return DomainConfigInstance(self._version, payload, domain_sid=self._solution['domain_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Messaging.V1.DomainConfigContext {}>'.format(context)


class DomainConfigInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, payload, domain_sid=None):
        """
        Initialize the DomainConfigInstance

        :returns: twilio.rest.messaging.v1.domain_config.DomainConfigInstance
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigInstance
        """
        super(DomainConfigInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'domain_sid': payload.get('domain_sid'),
            'config_sid': payload.get('config_sid'),
            'messaging_service_sids': payload.get('messaging_service_sids'),
            'fallback_url': payload.get('fallback_url'),
            'callback_url': payload.get('callback_url'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'domain_sid': domain_sid or self._properties['domain_sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: DomainConfigContext for this DomainConfigInstance
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigContext
        """
        if self._context is None:
            self._context = DomainConfigContext(self._version, domain_sid=self._solution['domain_sid'], )
        return self._context

    @property
    def domain_sid(self):
        """
        :returns: The unique string that we created to identify the Domain resource.
        :rtype: unicode
        """
        return self._properties['domain_sid']

    @property
    def config_sid(self):
        """
        :returns: The unique string that we created to identify the Domain config (prefix ZK).
        :rtype: unicode
        """
        return self._properties['config_sid']

    @property
    def messaging_service_sids(self):
        """
        :returns: A list of messagingServiceSids (with prefix MG).
        :rtype: list[unicode]
        """
        return self._properties['messaging_service_sids']

    @property
    def fallback_url(self):
        """
        :returns: We will redirect requests to urls we are unable to identify to this url.
        :rtype: unicode
        """
        return self._properties['fallback_url']

    @property
    def callback_url(self):
        """
        :returns: URL to receive click events to your webhook whenever the recipients click on the shortened links.
        :rtype: unicode
        """
        return self._properties['callback_url']

    @property
    def date_created(self):
        """
        :returns: Date this Domain Config was created.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: Date that this Domain Config was last updated.
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def update(self, messaging_service_sids, fallback_url=values.unset,
               callback_url=values.unset,
               messaging_service_sids_action=values.unset):
        """
        Update the DomainConfigInstance

        :param list[unicode] messaging_service_sids: A list of messagingServiceSids (with prefix MG)
        :param unicode fallback_url: We will redirect requests to urls we are unable to identify to this url.
        :param unicode callback_url: URL to receive click events to your webhook whenever the recipients click on the shortened links
        :param unicode messaging_service_sids_action: An action type for messaging_service_sids operation (ADD, DELETE, REPLACE)

        :returns: The updated DomainConfigInstance
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigInstance
        """
        return self._proxy.update(
            messaging_service_sids,
            fallback_url=fallback_url,
            callback_url=callback_url,
            messaging_service_sids_action=messaging_service_sids_action,
        )

    def fetch(self):
        """
        Fetch the DomainConfigInstance

        :returns: The fetched DomainConfigInstance
        :rtype: twilio.rest.messaging.v1.domain_config.DomainConfigInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Messaging.V1.DomainConfigInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class DomainCertsList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the DomainCertsList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.messaging.v1.domain_cert.DomainCertsList
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsList
        """
        super(DomainCertsList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, domain_sid):
        """
        Constructs a DomainCertsContext

        :param domain_sid: Unique string used to identify the domain that this certificate should be associated with.

        :returns: twilio.rest.messaging.v1.domain_cert.DomainCertsContext
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsContext
        """
        return DomainCertsContext(self._version, domain_sid=domain_sid, )

    def __call__(self, domain_sid):
        """
        Constructs a DomainCertsContext

        :param domain_sid: Unique string used to identify the domain that this certificate should be associated with.

        :returns: twilio.rest.messaging.v1.domain_cert.DomainCertsContext
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsContext
        """
        return DomainCertsContext(self._version, domain_sid=domain_sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1.DomainCertsList>'


class DomainCertsPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the DomainCertsPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.messaging.v1.domain_cert.DomainCertsPage
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsPage
        """
        super(DomainCertsPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of DomainCertsInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.messaging.v1.domain_cert.DomainCertsInstance
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsInstance
        """
        return DomainCertsInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1.DomainCertsPage>'


class DomainCertsContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, domain_sid):
        """
        Initialize the DomainCertsContext

        :param Version version: Version that contains the resource
        :param domain_sid: Unique string used to identify the domain that this certificate should be associated with.

        :returns: twilio.rest.messaging.v1.domain_cert.DomainCertsContext
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsContext
        """
        super(DomainCertsContext, self).__init__(version)

        # Path Solution
        self._solution = {'domain_sid': domain_sid, }
        self._uri = '/LinkShortening/Domains/{domain_sid}/Certificate'.format(**self._solution)

    def update(self, tls_cert):
        """
        Update the DomainCertsInstance

        :param unicode tls_cert: Certificate and private key information for this domain.

        :returns: The updated DomainCertsInstance
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsInstance
        """
        data = values.of({'TlsCert': tls_cert, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return DomainCertsInstance(self._version, payload, domain_sid=self._solution['domain_sid'], )

    def fetch(self):
        """
        Fetch the DomainCertsInstance

        :returns: The fetched DomainCertsInstance
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return DomainCertsInstance(self._version, payload, domain_sid=self._solution['domain_sid'], )

    def delete(self):
        """
        Deletes the DomainCertsInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Messaging.V1.DomainCertsContext {}>'.format(context)


class DomainCertsInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, payload, domain_sid=None):
        """
        Initialize the DomainCertsInstance

        :returns: twilio.rest.messaging.v1.domain_cert.DomainCertsInstance
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsInstance
        """
        super(DomainCertsInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'domain_sid': payload.get('domain_sid'),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'date_expires': deserialize.iso8601_datetime(payload.get('date_expires')),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'domain_name': payload.get('domain_name'),
            'certificate_sid': payload.get('certificate_sid'),
            'url': payload.get('url'),
            'validated': payload.get('validated'),
        }

        # Context
        self._context = None
        self._solution = {'domain_sid': domain_sid or self._properties['domain_sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: DomainCertsContext for this DomainCertsInstance
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsContext
        """
        if self._context is None:
            self._context = DomainCertsContext(self._version, domain_sid=self._solution['domain_sid'], )
        return self._context

    @property
    def domain_sid(self):
        """
        :returns: The unique string that we created to identify the Domain resource.
        :rtype: unicode
        """
        return self._properties['domain_sid']

    @property
    def date_updated(self):
        """
        :returns: Date that this Domain was last updated.
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def date_expires(self):
        """
        :returns: Expiration date for your private certificate.
        :rtype: datetime
        """
        return self._properties['date_expires']

    @property
    def date_created(self):
        """
        :returns: Date this Domain SID was created.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def domain_name(self):
        """
        :returns: Full url path for this domain.
        :rtype: unicode
        """
        return self._properties['domain_name']

    @property
    def certificate_sid(self):
        """
        :returns: The unique string that we created to identify this Certificate resource.
        :rtype: unicode
        """
        return self._properties['certificate_sid']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def validated(self):
        """
        :returns: Certificate validation field
        :rtype: bool
        """
        return self._properties['validated']

    def update(self, tls_cert):
        """
        Update the DomainCertsInstance

        :param unicode tls_cert: Certificate and private key information for this domain.

        :returns: The updated DomainCertsInstance
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsInstance
        """
        return self._proxy.update(tls_cert, )

    def fetch(self):
        """
        Fetch the DomainCertsInstance

        :returns: The fetched DomainCertsInstance
        :rtype: twilio.rest.messaging.v1.domain_cert.DomainCertsInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the DomainCertsInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Messaging.V1.DomainCertsInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class InsightsQuestionnairesQuestionList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version):
        """
        Initialize the InsightsQuestionnairesQuestionList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionList
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionList
        """
        super(InsightsQuestionnairesQuestionList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Insights/QM/Questions'.format(**self._solution)

    def create(self, category_id, question, description, answer_set_id, allow_na,
               token=values.unset):
        """
        Create the InsightsQuestionnairesQuestionInstance

        :param unicode category_id: Category ID
        :param unicode question: The question.
        :param unicode description: The question description.
        :param unicode answer_set_id: The answer_set for question.
        :param bool allow_na: Flag to enable NA for answer.
        :param unicode token: The Token HTTP request header

        :returns: The created InsightsQuestionnairesQuestionInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionInstance
        """
        data = values.of({
            'CategoryId': category_id,
            'Question': question,
            'Description': description,
            'AnswerSetId': answer_set_id,
            'AllowNa': allow_na,
        })
        headers = values.of({'Token': token, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, headers=headers, )

        return InsightsQuestionnairesQuestionInstance(self._version, payload, )

    def get(self, question_id):
        """
        Constructs a InsightsQuestionnairesQuestionContext

        :param question_id: Unique Question ID

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionContext
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionContext
        """
        return InsightsQuestionnairesQuestionContext(self._version, question_id=question_id, )

    def __call__(self, question_id):
        """
        Constructs a InsightsQuestionnairesQuestionContext

        :param question_id: Unique Question ID

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionContext
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionContext
        """
        return InsightsQuestionnairesQuestionContext(self._version, question_id=question_id, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InsightsQuestionnairesQuestionList>'


class InsightsQuestionnairesQuestionPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the InsightsQuestionnairesQuestionPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionPage
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionPage
        """
        super(InsightsQuestionnairesQuestionPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of InsightsQuestionnairesQuestionInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionInstance
        """
        return InsightsQuestionnairesQuestionInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InsightsQuestionnairesQuestionPage>'


class InsightsQuestionnairesQuestionContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, question_id):
        """
        Initialize the InsightsQuestionnairesQuestionContext

        :param Version version: Version that contains the resource
        :param question_id: Unique Question ID

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionContext
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionContext
        """
        super(InsightsQuestionnairesQuestionContext, self).__init__(version)

        # Path Solution
        self._solution = {'question_id': question_id, }
        self._uri = '/Insights/QM/Questions/{question_id}'.format(**self._solution)

    def update(self, question, description, answer_set_id, allow_na,
               category_id=values.unset, token=values.unset):
        """
        Update the InsightsQuestionnairesQuestionInstance

        :param unicode question: The question.
        :param unicode description: The question description.
        :param unicode answer_set_id: The answer_set for question.
        :param bool allow_na: Flag to enable NA for answer.
        :param unicode category_id: Category ID
        :param unicode token: The Token HTTP request header

        :returns: The updated InsightsQuestionnairesQuestionInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionInstance
        """
        data = values.of({
            'Question': question,
            'Description': description,
            'AnswerSetId': answer_set_id,
            'AllowNa': allow_na,
            'CategoryId': category_id,
        })
        headers = values.of({'Token': token, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, headers=headers, )

        return InsightsQuestionnairesQuestionInstance(
            self._version,
            payload,
            question_id=self._solution['question_id'],
        )

    def delete(self, token=values.unset):
        """
        Deletes the InsightsQuestionnairesQuestionInstance

        :param unicode token: The Token HTTP request header

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        headers = values.of({'Token': token, })

        return self._version.delete(method='DELETE', uri=self._uri, headers=headers, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InsightsQuestionnairesQuestionContext {}>'.format(context)


class InsightsQuestionnairesQuestionInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload, question_id=None):
        """
        Initialize the InsightsQuestionnairesQuestionInstance

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionInstance
        """
        super(InsightsQuestionnairesQuestionInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'question_id': payload.get('question_id'),
            'question': payload.get('question'),
            'description': payload.get('description'),
            'category': payload.get('category'),
            'answer_set_id': payload.get('answer_set_id'),
            'allow_na': payload.get('allow_na'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'question_id': question_id or self._properties['question_id'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: InsightsQuestionnairesQuestionContext for this InsightsQuestionnairesQuestionInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionContext
        """
        if self._context is None:
            self._context = InsightsQuestionnairesQuestionContext(
                self._version,
                question_id=self._solution['question_id'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource and owns this Flex Insights
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def question_id(self):
        """
        :returns: Unique Question ID
        :rtype: unicode
        """
        return self._properties['question_id']

    @property
    def question(self):
        """
        :returns: The question.
        :rtype: unicode
        """
        return self._properties['question']

    @property
    def description(self):
        """
        :returns: The question description.
        :rtype: unicode
        """
        return self._properties['description']

    @property
    def category(self):
        """
        :returns: The question category.
        :rtype: dict
        """
        return self._properties['category']

    @property
    def answer_set_id(self):
        """
        :returns: The answer_set for question.
        :rtype: unicode
        """
        return self._properties['answer_set_id']

    @property
    def allow_na(self):
        """
        :returns: Flag to enable NA for answer.
        :rtype: bool
        """
        return self._properties['allow_na']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def update(self, question, description, answer_set_id, allow_na,
               category_id=values.unset, token=values.unset):
        """
        Update the InsightsQuestionnairesQuestionInstance

        :param unicode question: The question.
        :param unicode description: The question description.
        :param unicode answer_set_id: The answer_set for question.
        :param bool allow_na: Flag to enable NA for answer.
        :param unicode category_id: Category ID
        :param unicode token: The Token HTTP request header

        :returns: The updated InsightsQuestionnairesQuestionInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_question.InsightsQuestionnairesQuestionInstance
        """
        return self._proxy.update(
            question,
            description,
            answer_set_id,
            allow_na,
            category_id=category_id,
            token=token,
        )

    def delete(self, token=values.unset):
        """
        Deletes the InsightsQuestionnairesQuestionInstance

        :param unicode token: The Token HTTP request header

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete(token=token, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InsightsQuestionnairesQuestionInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class InsightsQuestionnairesCategoryList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version):
        """
        Initialize the InsightsQuestionnairesCategoryList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryList
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryList
        """
        super(InsightsQuestionnairesCategoryList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Insights/QM/Categories'.format(**self._solution)

    def create(self, name, token=values.unset):
        """
        Create the InsightsQuestionnairesCategoryInstance

        :param unicode name: The category name.
        :param unicode token: The Token HTTP request header

        :returns: The created InsightsQuestionnairesCategoryInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryInstance
        """
        data = values.of({'Name': name, })
        headers = values.of({'Token': token, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, headers=headers, )

        return InsightsQuestionnairesCategoryInstance(self._version, payload, )

    def get(self, category_id):
        """
        Constructs a InsightsQuestionnairesCategoryContext

        :param category_id: Category ID to update

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryContext
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryContext
        """
        return InsightsQuestionnairesCategoryContext(self._version, category_id=category_id, )

    def __call__(self, category_id):
        """
        Constructs a InsightsQuestionnairesCategoryContext

        :param category_id: Category ID to update

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryContext
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryContext
        """
        return InsightsQuestionnairesCategoryContext(self._version, category_id=category_id, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InsightsQuestionnairesCategoryList>'


class InsightsQuestionnairesCategoryPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the InsightsQuestionnairesCategoryPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryPage
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryPage
        """
        super(InsightsQuestionnairesCategoryPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of InsightsQuestionnairesCategoryInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryInstance
        """
        return InsightsQuestionnairesCategoryInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InsightsQuestionnairesCategoryPage>'


class InsightsQuestionnairesCategoryContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, category_id):
        """
        Initialize the InsightsQuestionnairesCategoryContext

        :param Version version: Version that contains the resource
        :param category_id: Category ID to update

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryContext
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryContext
        """
        super(InsightsQuestionnairesCategoryContext, self).__init__(version)

        # Path Solution
        self._solution = {'category_id': category_id, }
        self._uri = '/Insights/QM/Categories/{category_id}'.format(**self._solution)

    def update(self, name, token=values.unset):
        """
        Update the InsightsQuestionnairesCategoryInstance

        :param unicode name: The category name.
        :param unicode token: The Token HTTP request header

        :returns: The updated InsightsQuestionnairesCategoryInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryInstance
        """
        data = values.of({'Name': name, })
        headers = values.of({'Token': token, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, headers=headers, )

        return InsightsQuestionnairesCategoryInstance(
            self._version,
            payload,
            category_id=self._solution['category_id'],
        )

    def delete(self, token=values.unset):
        """
        Deletes the InsightsQuestionnairesCategoryInstance

        :param unicode token: The Token HTTP request header

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        headers = values.of({'Token': token, })

        return self._version.delete(method='DELETE', uri=self._uri, headers=headers, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InsightsQuestionnairesCategoryContext {}>'.format(context)


class InsightsQuestionnairesCategoryInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload, category_id=None):
        """
        Initialize the InsightsQuestionnairesCategoryInstance

        :returns: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryInstance
        """
        super(InsightsQuestionnairesCategoryInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'category_id': payload.get('category_id'),
            'name': payload.get('name'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'category_id': category_id or self._properties['category_id'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: InsightsQuestionnairesCategoryContext for this InsightsQuestionnairesCategoryInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryContext
        """
        if self._context is None:
            self._context = InsightsQuestionnairesCategoryContext(
                self._version,
                category_id=self._solution['category_id'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource and owns this Flex Insights
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def category_id(self):
        """
        :returns: Unique category ID
        :rtype: unicode
        """
        return self._properties['category_id']

    @property
    def name(self):
        """
        :returns: The category name.
        :rtype: unicode
        """
        return self._properties['name']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def update(self, name, token=values.unset):
        """
        Update the InsightsQuestionnairesCategoryInstance

        :param unicode name: The category name.
        :param unicode token: The Token HTTP request header

        :returns: The updated InsightsQuestionnairesCategoryInstance
        :rtype: twilio.rest.flex_api.v1.insights_questionnaires_category.InsightsQuestionnairesCategoryInstance
        """
        return self._proxy.update(name, token=token, )

    def delete(self, token=values.unset):
        """
        Deletes the InsightsQuestionnairesCategoryInstance

        :param unicode token: The Token HTTP request header

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete(token=token, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InsightsQuestionnairesCategoryInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class WebChannelsList(ListResource):

    def __init__(self, version):
        """
        Initialize the WebChannelsList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v2.web_channels.WebChannelsList
        :rtype: twilio.rest.flex_api.v2.web_channels.WebChannelsList
        """
        super(WebChannelsList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/WebChats'.format(**self._solution)

    def create(self, address_sid, chat_friendly_name=values.unset,
               customer_friendly_name=values.unset,
               pre_engagement_data=values.unset):
        """
        Create the WebChannelsInstance

        :param unicode address_sid: The SID of the Conversations Address
        :param unicode chat_friendly_name: The Conversation's friendly name
        :param unicode customer_friendly_name: The Conversation participant's friendly name
        :param unicode pre_engagement_data: The pre-engagement data

        :returns: The created WebChannelsInstance
        :rtype: twilio.rest.flex_api.v2.web_channels.WebChannelsInstance
        """
        data = values.of({
            'AddressSid': address_sid,
            'ChatFriendlyName': chat_friendly_name,
            'CustomerFriendlyName': customer_friendly_name,
            'PreEngagementData': pre_engagement_data,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return WebChannelsInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V2.WebChannelsList>'


class WebChannelsPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the WebChannelsPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.flex_api.v2.web_channels.WebChannelsPage
        :rtype: twilio.rest.flex_api.v2.web_channels.WebChannelsPage
        """
        super(WebChannelsPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of WebChannelsInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v2.web_channels.WebChannelsInstance
        :rtype: twilio.rest.flex_api.v2.web_channels.WebChannelsInstance
        """
        return WebChannelsInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V2.WebChannelsPage>'


class WebChannelsInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the WebChannelsInstance

        :returns: twilio.rest.flex_api.v2.web_channels.WebChannelsInstance
        :rtype: twilio.rest.flex_api.v2.web_channels.WebChannelsInstance
        """
        super(WebChannelsInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'conversation_sid': payload.get('conversation_sid'),
            'identity': payload.get('identity'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def conversation_sid(self):
        """
        :returns: The unique string representing the Conversation resource created
        :rtype: unicode
        """
        return self._properties['conversation_sid']

    @property
    def identity(self):
        """
        :returns: The unique string representing the User created
        :rtype: unicode
        """
        return self._properties['identity']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V2.WebChannelsInstance>'
